// ShowItemIcon.js Ver.1.0.3
// MIT License (C) 2022 あわやまたな
// http://opensource.org/licenses/mit-license.php

/*:
* @target MV MZ
* @plugindesc  キャラクターの頭上にアイテムアイコンを表示できます。
* @author あわやまたな (Awaya_Matana)
* @url https://awaya3ji.seesaa.net/
* @help［プラグインコマンド（MV）］
* showItemIcon イベントID アイコン番号
* アイコン番号が未入力、または0の時にアイコンを消去できます。
*
* ［スクリプト］
* 【イベントエディター】
* this.character(イベントID).setIcon(アイコン番号)
*
* 【移動ルートの設定】
* this.setIcon(アイコン番号)
* 両方ともアイコン番号が未入力、または0の時にアイコンを消去できます。
*
*［メモ］
* イベントのメモ欄にアイコン番号をコンマ区切りで入力することでページ毎に
* 頭上に表示するアイテムアイコンの
* 初期状態を設定できます。
* 設定しない場所は0で埋めるか空欄にして下さい。
* <itemIcon:50> //1ページ目をアイコン50にする
* <itemIcon:0,0,120> <itemIcon:,,120> //3ページ目をアイコン120にする
* <itemIcon:50,0,120> <itemIcon:50,,120> //1ページ目をアイコン50、3ページ目をアイコン120にする
*
*［Z座標］
* 0：下層タイル
* 1：下層キャラクター
* 3：通常キャラクター
* 4：上層タイル
* 5：上層キャラクター
* 6：飛行船の影
* 7：フキダシアイコン
* 8：アニメーション
* 9：タップエフェクト
*
* [更新履歴]
* 2022/01/24：Ver.1.0.0　公開
* 2022/01/24：Ver.1.0.1　イベントの一時消去時にアイコンが残留する不具合を修正。
* 2022/01/24：Ver.1.0.2　スクリプトで入力しやすいように関数を追加。
* 2022/01/24：Ver.1.0.3　アイコンの寸法取得方法を変更。
*
* @command showItemIcon
* @text アイテムアイコンの表示
* @desc キャラクターの頭上に、指定したアイテムアイコンを表示します。
*
* @arg character
* @text キャラクター
* @desc 対象とするキャラクターです。
* -1：プレイヤー　0：このイベント
* @default -1
* @type string
*
* @arg itemIcon
* @text アイテムアイコン
* @desc 表示するアイテムアイコンのインデックスです。
* 未入力、または0で消去。
* @default 
* @type string
*
* @param iconY
* @text アイコンY座標
* @desc アイコンのY座標を設定値分ずらして表示します。
* @default 0
* @type number
* @min -999999
*
* @param iconZ
* @text アイコンZ座標
* @desc アイコンのZ座標を設定します。
* @default 6.5
* @type string
*
*/

'use strict';
{
	const useMZ = Utils.RPGMAKER_NAME === "MZ";
	const pluginName = document.currentScript.src.replace(/^.*\/(.*).js$/, (_, p1) => p1);
	const hasPluginCommonBase = $plugins.some(plugin => plugin.name === "PluginCommonBase" && plugin.status);
	const parameter = PluginManager.parameters(pluginName);
	const iconY = Number(parameter['iconY']);
	const iconZ = Number(parameter['iconZ']);
	
	if (useMZ && hasPluginCommonBase) {
		PluginManagerEx.registerCommand(document.currentScript, "showItemIcon", function (args) {
			this.character(args.character).setIcon(args.itemIcon);
		});
	} else if (useMZ) {
		PluginManager.registerCommand(pluginName, "showItemIcon", function (args) {
			this.character(+args.character).setIcon(+args.itemIcon);
		});
	}

	const _Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
	Game_Interpreter.prototype.pluginCommand = function(command, args) {
		_Game_Interpreter_pluginCommand.apply(this, arguments);
		
		if (command === "showItemIcon") {
			this.character(+args[0]).setIcon(+args[1]);
		}	
		
	};

	//-----------------------------------------------------------------------------
	// Sprite_ItemIcon

	function Sprite_ItemIcon() {
		this.initialize(...arguments);
	}

	Sprite_ItemIcon.prototype = Object.create(Sprite.prototype);
	Sprite_ItemIcon.prototype.constructor = Sprite_ItemIcon;

	Sprite_ItemIcon.prototype.initialize = function() {
		Sprite.prototype.initialize.call(this);
		this.initMembers();
		this.loadBitmap();
	};

	Sprite_ItemIcon.prototype.initMembers = function() {
		this._target = null;
		this._itemIconId = 0;
		this.anchor.x = 0.5;
		this.anchor.y = 1;
		this.z = iconZ;
	};

	Sprite_ItemIcon.prototype.loadBitmap = function() {
		this.bitmap = ImageManager.loadSystem("IconSet");
		this.setFrame(0, 0, 0, 0);
	};

	Sprite_ItemIcon.prototype.setup = function(targetSprite, itemIconId) {
		this._target = targetSprite;
		this._itemIconId = itemIconId;
		this.updateFrame();
	};

	Sprite_ItemIcon.prototype.update = function() {
		Sprite.prototype.update.call(this);
		this.updatePosition();
	};

	Sprite_ItemIcon.prototype.updatePosition = function() {
		this.x = this._target.x;
		this.y = this._target.y + iconY - (this._target._character._easyRotating ? this._target.height /2 : this._target.height);
	};

	Sprite_ItemIcon.prototype.updateFrame = function() {
		const w = useMZ ? ImageManager.iconWidth : Window_Base._iconWidth;
		const h = useMZ ? ImageManager.iconHeight : Window_Base._iconHeight;
		const sx = this._itemIconId %16 * w;
		const sy = Math.floor(this._itemIconId /16) * h;
		this.setFrame(sx, sy, w, h);
	};

	//-----------------------------------------------------------------------------
	// Spriteset_Map

	const _Spriteset_Map_initialize = Spriteset_Map.prototype.initialize;
	Spriteset_Map.prototype.initialize = function() {
		_Spriteset_Map_initialize.call(this);
		if (useMZ) this._itemIconSprites = [];
		this.restoreItemIcons();
	};
	const setItemIcon = useMZ ? "setItemIcon" : "setItemIconMV";
	//マップ復帰時に復元
	Spriteset_Map.prototype.restoreItemIcons = function() {
		for (const sprite of this._characterSprites) {
			const character = sprite._character;
			const itemIconId = character.itemIconId()
			this[setItemIcon]({target:character, itemIconId:itemIconId});
		}
	};
	//MZ
	const _Spriteset_Map_destroy = Spriteset_Map.prototype.destroy;
	Spriteset_Map.prototype.destroy = function(options) {
		this.removeAllItemIcons();
		_Spriteset_Map_destroy.call(this, options);
	};
	//MZ
	const _Spriteset_Map_loadSystemImages = Spriteset_Map.prototype.loadSystemImages;
	Spriteset_Map.prototype.loadSystemImages = function() {
		_Spriteset_Map_loadSystemImages.call(this);
		ImageManager.loadSystem("IconSet");
	};

	const _Spriteset_Map_update = Spriteset_Map.prototype.update;
	Spriteset_Map.prototype.update = function() {
		_Spriteset_Map_update.call(this);
		this.updateItemIcons();
	};

	Spriteset_Map.prototype.updateItemIcons = function() {
		this.processItemIconRequests();
	};

	Spriteset_Map.prototype.processItemIconRequests = function() {
		for (;;) {
			const request = $gameTemp.retrieveItemIcon();
			if (request) {
				this[setItemIcon](request);
			} else {
				break;
			}
		}
	};
	//MZ
	Spriteset_Map.prototype.setItemIcon = function(request) {
		const targetSprite = this.findTargetSprite(request.target);
		if (targetSprite) {
			//アイコンが既に表示されている場合は既存のスプライトに変更を加えるだけで終了
			const existingItemIcon = this.existingItemIcon(request);
			if (existingItemIcon) {
				if (request.itemIconId > 0) {
					existingItemIcon.setup(targetSprite, request.itemIconId);
				} else {
					//アイコン番号が0なら破棄
					this.removeItemIcon(existingItemIcon);
				}
			} else if (request.itemIconId > 0) {
				//アイコン番号が0より大きければスプライトを生成
				const sprite = new Sprite_ItemIcon();
				sprite.targetObject = request.target;
				sprite.setup(targetSprite, request.itemIconId);
				this._effectsContainer.addChild(sprite);
				this._itemIconSprites.push(sprite);
			}
			
		}
	};
	//MZ //既存のアイコンが存在するかどうか
	Spriteset_Map.prototype.existingItemIcon = function(request) {
		for (const sprite of this._itemIconSprites) {
			if (sprite.targetObject === request.target) {
				return sprite;
			}
		}
	};
	//MZ
	Spriteset_Map.prototype.removeItemIcon = function(sprite) {
		this._itemIconSprites.remove(sprite);
		this._effectsContainer.removeChild(sprite);
		sprite.destroy();
	};
	//MZ
	Spriteset_Map.prototype.removeAllItemIcons = function() {
		for (const sprite of this._itemIconSprites.clone()) {
			this.removeItemIcon(sprite);
		}
	};
	//MV
	Spriteset_Map.prototype.setItemIconMV = function(request) {
		const targetSprite = this.findTargetSpriteMV(request.target);
		if (targetSprite) {
			if(request.itemIconId > 0) {
				targetSprite.createItemIconMV();
			} else {
				targetSprite.removeItemIconMV();
			}
		}
	};
	//MV
	Spriteset_Map.prototype.findTargetSpriteMV = function(target) {
		return this._characterSprites.find(sprite => sprite._character === target);
	};

	//-----------------------------------------------------------------------------
	// Sprite_Character

	Sprite_Character.prototype.createItemIconMV = function() {
		if (!this._itemIconSprite) this._itemIconSprite = new Sprite_ItemIcon();
		this._itemIconSprite.setup(this, this._character.itemIconId());
		this.parent.addChild(this._itemIconSprite);
	};

	Sprite_Character.prototype.removeItemIconMV = function() {
		if (this._itemIconSprite) {
			this.parent.removeChild(this._itemIconSprite);
			this._itemIconSprite = null;
		}
	};
	
	//-----------------------------------------------------------------------------
	// Game_Temp

	const _Game_Temp_initialize = Game_Temp.prototype.initialize;
	Game_Temp.prototype.initialize = function() {
		_Game_Temp_initialize.call(this);
		this._itemIconQueue = [];
	};

	Game_Temp.prototype.requestItemIcon = function(target, itemIconId = 0) {
		const request = { target: target, itemIconId: itemIconId };
		target.setItemIconId(itemIconId);
		this._itemIconQueue.push(request);
	};

	Game_Temp.prototype.retrieveItemIcon = function() {
		return this._itemIconQueue.shift();
	};

	//-----------------------------------------------------------------------------
	// Game_CharacterBase

	const _Game_CharacterBase_initMembers = Game_CharacterBase.prototype.initMembers;
	Game_CharacterBase.prototype.initMembers = function() {
		_Game_CharacterBase_initMembers.call(this);
		this._itemIconId = 0;
	};

	Game_CharacterBase.prototype.setIcon = function (itemIconId){
		$gameTemp.requestItemIcon(this,itemIconId);
	};

	Game_CharacterBase.prototype.setItemIconId = function(itemIconId) {
		this._itemIconId = itemIconId;
	};
	
	Game_CharacterBase.prototype.itemIconId = function() {
		return this._itemIconId;
	};

	//-----------------------------------------------------------------------------
	// Game_Event

	const _Game_Event_clearPageSettings = Game_Event.prototype.clearPageSettings;
	Game_Event.prototype.clearPageSettings = function() {
		_Game_Event_clearPageSettings.call(this);
		this.setIcon();
	};
	
	const _Game_Event_setupPageSettings = Game_Event.prototype.setupPageSettings;
	Game_Event.prototype.setupPageSettings = function() {
		_Game_Event_setupPageSettings.call(this);
		this.setupSettingsItemIcon();
	};
	
	Game_Event.prototype.setupSettingsItemIcon = function() {
		const page = this.page();
		this.setIcon(page.itemIcon);
	};

	//-----------------------------------------------------------------------------
	// DataManager
	//マップデータロード時の処理に追加
	const _DataManager_onLoad = DataManager.onLoad;
	DataManager.onLoad = function(object) {
		_DataManager_onLoad.call(this,object);
		if (!!(object.data && object.events)) {
			this.addItemIconToCondition();
		}
	};
	//メモ欄の情報を事前に解析し、$dataMapに書き込むことでリフレッシュ時の負担を軽減。実質的にイベントのオプションと同じ仕様にする。
	DataManager.addItemIconToCondition = function() {
		$dataMap.events.forEach( data => {
			if(!data) return;
			const pages = data.pages;
			const meta  = data.meta['itemIcon'];
			const itemIconPages = this.analyticsItemIcon(pages, meta);
			for (let i=0;i<=pages.length-1;i++) {
				pages[i]['itemIcon'] = itemIconPages[i];
			}
		});
	};
	//解析
	DataManager.analyticsItemIcon = function(pages,meta) {
		if (typeof meta !== "boolean") {
			const itemIconPages = Array(pages.length).fill(0);
			if (meta) {
				const arrMeta   = meta.split(',').map(Number);
				for (let i=0;arrMeta.length-1>=i;i++) {
					itemIconPages[i] = arrMeta[i];
				}
			}
			return itemIconPages;
		}
	};

}